@@ -5,9 +5,21 @@ use vars qw(@ISA);
 
 sub echo {
   my $self = shift;
+  pop; # last parameter is envelope (SOAP::SOM object)
+       # don't want to echo it
   @_;
 }
 
+sub echodata {
+  my $self = shift;
+  my @parameters = pop->dataof(SOAP::SOM::paramsin);
+  @parameters;
+}
+
+sub echotwo {
+  SOAP::Data->type(xml => "<a>$_[1]</a><b>$_[2]</b>");
+}
+
 sub autobind {
   my $self = shift;
   my $param1 = shift;
@@ -29,9 +41,14 @@ sub byorder {
 }
 
 sub byname { # input parameter(s), envelope (SOAP::SOM object)
-  # pop() will return SOAP::SOM object
-  # SOM->method will return structure with parameters {name => value, ...}
-  my($a, $b, $c) = @{pop->method}{qw(a b c)};
+  my $self = shift;
+  my($a, $b, $c) = SOAP::Server::Parameters::byName([qw(a b c)], @_);
+  return "a=$a, b=$b, c=$c";
+}
+
+sub bynameororder { # input parameter(s), envelope (SOAP::SOM object)
+  my $self = shift;
+  my($a, $b, $c) = SOAP::Server::Parameters::byNameOrOrder([qw(a b c)], @_);
   return "a=$a, b=$b, c=$c";
 }
 
@@ -43,4 +60,11 @@ sub die_with_object {
   die SOAP::Data->name(something => 'value')->uri('http://www.soaplite.com/');
 }
 
+sub die_with_fault {
+  die SOAP::Fault->faultcode('Server.Custom') # will be qualified
+                 ->faultstring('Died in server method')
+                 ->faultdetail(bless {code => 1} => 'BadError')
+                 ->faultactor('http://www.soaplite.com/custom');
+}
+
 1;
\ No newline at end of file
